@@ -2,7 +2,7 @@
 This script is part of the pytest release process which is triggered by comments
 in issues.
 
-This script is started by the `prepare_release.yml` workflow, which is triggered by two comment
+This script is started by the `release-on-comment.yml` workflow, which is triggered by two comment
 related events:
 
 * https://help.github.com/en/actions/reference/events-that-trigger-workflows#issue-comment-event-issue_comment
@@ -10,12 +10,13 @@ related events:
 
 This script receives the payload and a secrets on the command line.
 
-The payload must contain a comment with a phrase matching this regular expression:
+The payload must contain a comment with a phrase matching this pseudo-regular expression:
 
-    @pytestbot please prepare release from <branch name>
+    @pytestbot please prepare (major )? release from <branch name>
 
 Then the appropriate version will be obtained based on the given branch name:
 
+* a major release from master if "major" appears in the phrase in that position
 * a feature or bug fix release from master (based if there are features in the current changelog
   folder)
 * a bug fix from a maintenance branch
@@ -31,8 +32,10 @@ import os
 import re
 import sys
 from pathlib import Path
+from subprocess import CalledProcessError
 from subprocess import check_call
 from subprocess import check_output
+from subprocess import run
 from textwrap import dedent
 from typing import Dict
 from typing import Optional
@@ -74,15 +77,15 @@ def get_comment_data(payload: Dict) -> str:
 
 def validate_and_get_issue_comment_payload(
     issue_payload_path: Optional[Path],
-) -> Tuple[str, str]:
+) -> Tuple[str, str, bool]:
     payload = json.loads(issue_payload_path.read_text(encoding="UTF-8"))
     body = get_comment_data(payload)["body"]
-    m = re.match(r"@pytestbot please prepare release from ([\w\-_\.]+)", body)
+    m = re.match(r"@pytestbot please prepare (major )?release from ([\w\-_\.]+)", body)
     if m:
-        base_branch = m.group(1)
+        is_major, base_branch = m.group(1) is not None, m.group(2)
     else:
-        base_branch = None
-    return payload, base_branch
+        is_major, base_branch = False, None
+    return payload, base_branch, is_major
 
 
 def print_and_exit(msg) -> None:
@@ -91,7 +94,10 @@ def print_and_exit(msg) -> None:
 
 
 def trigger_release(payload_path: Path, token: str) -> None:
-    payload, base_branch = validate_and_get_issue_comment_payload(payload_path)
+    error_contents = ""  # to be used to store error output in case any command fails
+    payload, base_branch, is_major = validate_and_get_issue_comment_payload(
+        payload_path
+    )
     if base_branch is None:
         url = get_comment_data(payload)["html_url"]
         print_and_exit(
@@ -106,10 +112,9 @@ def trigger_release(payload_path: Path, token: str) -> None:
     issue = repo.issue(issue_number)
 
     check_call(["git", "checkout", f"origin/{base_branch}"])
-    print("DEBUG:", check_output(["git", "rev-parse", "HEAD"]))
 
     try:
-        version = find_next_version(base_branch)
+        version = find_next_version(base_branch, is_major)
     except InvalidFeatureRelease as e:
         issue.create_comment(str(e))
         print_and_exit(f"{Fore.RED}{e}")
@@ -119,19 +124,42 @@ def trigger_release(payload_path: Path, token: str) -> None:
 
         release_branch = f"release-{version}"
 
-        check_call(["git", "config", "user.name", "pytest bot"])
-        check_call(["git", "config", "user.email", "pytestbot@gmail.com"])
+        run(
+            ["git", "config", "user.name", "pytest bot"],
+            text=True,
+            check=True,
+            capture_output=True,
+        )
+        run(
+            ["git", "config", "user.email", "pytestbot@gmail.com"],
+            text=True,
+            check=True,
+            capture_output=True,
+        )
 
-        check_call(["git", "checkout", "-b", release_branch, f"origin/{base_branch}"])
+        run(
+            ["git", "checkout", "-b", release_branch, f"origin/{base_branch}"],
+            text=True,
+            check=True,
+            capture_output=True,
+        )
 
         print(f"Branch {Fore.CYAN}{release_branch}{Fore.RESET} created.")
 
-        check_call(
-            [sys.executable, "scripts/release.py", version, "--skip-check-links"]
+        run(
+            [sys.executable, "scripts/release.py", version, "--skip-check-links"],
+            text=True,
+            check=True,
+            capture_output=True,
         )
 
         oauth_url = f"https://{token}:x-oauth-basic@github.com/{SLUG}.git"
-        check_call(["git", "push", oauth_url, f"HEAD:{release_branch}", "--force"])
+        run(
+            ["git", "push", oauth_url, f"HEAD:{release_branch}", "--force"],
+            text=True,
+            check=True,
+            capture_output=True,
+        )
         print(f"Branch {Fore.CYAN}{release_branch}{Fore.RESET} pushed.")
 
         body = PR_BODY.format(
@@ -151,7 +179,10 @@ def trigger_release(payload_path: Path, token: str) -> None:
         print(f"Notified in original comment {Fore.CYAN}{comment.url}{Fore.RESET}.")
 
         print(f"{Fore.GREEN}Success.")
+    except CalledProcessError as e:
+        error_contents = e.output
     except Exception as e:
+        error_contents = str(e)
         link = f"https://github.com/{SLUG}/actions/runs/{os.environ['GITHUB_RUN_ID']}"
         issue.create_comment(
             dedent(
@@ -168,8 +199,25 @@ def trigger_release(payload_path: Path, token: str) -> None:
         )
         print_and_exit(f"{Fore.RED}{e}")
 
+    if error_contents:
+        link = f"https://github.com/{SLUG}/actions/runs/{os.environ['GITHUB_RUN_ID']}"
+        issue.create_comment(
+            dedent(
+                f"""
+                Sorry, the request to prepare release `{version}` from {base_branch} failed with:
 
-def find_next_version(base_branch: str) -> str:
+                ```
+                {error_contents}
+                ```
+
+                See: {link}.
+                """
+            )
+        )
+        print_and_exit(f"{Fore.RED}{error_contents}")
+
+
+def find_next_version(base_branch: str, is_major: bool) -> str:
     output = check_output(["git", "tag"], encoding="UTF-8")
     valid_versions = []
     for v in output.splitlines():
@@ -196,7 +244,9 @@ def find_next_version(base_branch: str) -> str:
         msg += "\n".join(f"* `{x.name}`" for x in sorted(features + breaking))
         raise InvalidFeatureRelease(msg)
 
-    if is_feature_release:
+    if is_major:
+        return f"{last_version[0]+1}.0.0"
+    elif is_feature_release:
         return f"{last_version[0]}.{last_version[1] + 1}.0"
     else:
         return f"{last_version[0]}.{last_version[1]}.{last_version[2] + 1}"
