@@ -1,8 +1,13 @@
 import io
 import os
 import re
+from typing import cast
 
 import pytest
+from _pytest.capture import CaptureManager
+from _pytest.config import ExitCode
+from _pytest.pytester import Testdir
+from _pytest.terminal import TerminalReporter
 
 
 def test_nothing_logged(testdir):
@@ -166,60 +171,6 @@ def test_teardown_logging(testdir):
     )
 
 
-def test_disable_log_capturing(testdir):
-    testdir.makepyfile(
-        """
-        import sys
-        import logging
-
-        logger = logging.getLogger(__name__)
-
-        def test_foo():
-            sys.stdout.write('text going to stdout')
-            logger.warning('catch me if you can!')
-            sys.stderr.write('text going to stderr')
-            assert False
-        """
-    )
-    result = testdir.runpytest("--no-print-logs")
-    print(result.stdout)
-    assert result.ret == 1
-    result.stdout.fnmatch_lines(["*- Captured stdout call -*", "text going to stdout"])
-    result.stdout.fnmatch_lines(["*- Captured stderr call -*", "text going to stderr"])
-    with pytest.raises(pytest.fail.Exception):
-        result.stdout.fnmatch_lines(["*- Captured *log call -*"])
-
-
-def test_disable_log_capturing_ini(testdir):
-    testdir.makeini(
-        """
-        [pytest]
-        log_print=False
-        """
-    )
-    testdir.makepyfile(
-        """
-        import sys
-        import logging
-
-        logger = logging.getLogger(__name__)
-
-        def test_foo():
-            sys.stdout.write('text going to stdout')
-            logger.warning('catch me if you can!')
-            sys.stderr.write('text going to stderr')
-            assert False
-        """
-    )
-    result = testdir.runpytest()
-    print(result.stdout)
-    assert result.ret == 1
-    result.stdout.fnmatch_lines(["*- Captured stdout call -*", "text going to stdout"])
-    result.stdout.fnmatch_lines(["*- Captured stderr call -*", "text going to stderr"])
-    with pytest.raises(pytest.fail.Exception):
-        result.stdout.fnmatch_lines(["*- Captured *log call -*"])
-
-
 @pytest.mark.parametrize("enabled", [True, False])
 def test_log_cli_enabled_disabled(testdir, enabled):
     msg = "critical message logged by test"
@@ -861,7 +812,7 @@ def test_log_file_unicode(testdir):
 
 
 @pytest.mark.parametrize("has_capture_manager", [True, False])
-def test_live_logging_suspends_capture(has_capture_manager, request):
+def test_live_logging_suspends_capture(has_capture_manager: bool, request) -> None:
     """Test that capture manager is suspended when we emitting messages for live logging.
 
     This tests the implementation calls instead of behavior because it is difficult/impossible to do it using
@@ -888,8 +839,10 @@ def test_live_logging_suspends_capture(has_capture_manager, request):
         def section(self, *args, **kwargs):
             pass
 
-    out_file = DummyTerminal()
-    capture_manager = MockCaptureManager() if has_capture_manager else None
+    out_file = cast(TerminalReporter, DummyTerminal())
+    capture_manager = (
+        cast(CaptureManager, MockCaptureManager()) if has_capture_manager else None
+    )
     handler = _LiveLoggingStreamHandler(out_file, capture_manager)
     handler.set_when("call")
 
@@ -902,7 +855,7 @@ def test_live_logging_suspends_capture(has_capture_manager, request):
         assert MockCaptureManager.calls == ["enter disabled", "exit disabled"]
     else:
         assert MockCaptureManager.calls == []
-    assert out_file.getvalue() == "\nsome message\n"
+    assert cast(io.StringIO, out_file).getvalue() == "\nsome message\n"
 
 
 def test_collection_live_logging(testdir):
@@ -1103,11 +1056,11 @@ def test_log_set_path(testdir):
         """
     )
     testdir.runpytest()
-    with open(os.path.join(report_dir_base, "test_first"), "r") as rfh:
+    with open(os.path.join(report_dir_base, "test_first")) as rfh:
         content = rfh.read()
         assert "message from test 1" in content
 
-    with open(os.path.join(report_dir_base, "test_second"), "r") as rfh:
+    with open(os.path.join(report_dir_base, "test_second")) as rfh:
         content = rfh.read()
         assert "message from test 2" in content
 
@@ -1155,3 +1108,56 @@ def test_colored_ansi_esc_caplogtext(testdir):
     )
     result = testdir.runpytest("--log-level=INFO", "--color=yes")
     assert result.ret == 0
+
+
+def test_logging_emit_error(testdir: Testdir) -> None:
+    """
+    An exception raised during emit() should fail the test.
+
+    The default behavior of logging is to print "Logging error"
+    to stderr with the call stack and some extra details.
+
+    pytest overrides this behavior to propagate the exception.
+    """
+    testdir.makepyfile(
+        """
+        import logging
+
+        def test_bad_log():
+            logging.warning('oops', 'first', 2)
+        """
+    )
+    result = testdir.runpytest()
+    result.assert_outcomes(failed=1)
+    result.stdout.fnmatch_lines(
+        [
+            "====* FAILURES *====",
+            "*not all arguments converted during string formatting*",
+        ]
+    )
+
+
+def test_logging_emit_error_supressed(testdir: Testdir) -> None:
+    """
+    If logging is configured to silently ignore errors, pytest
+    doesn't propagate errors either.
+    """
+    testdir.makepyfile(
+        """
+        import logging
+
+        def test_bad_log(monkeypatch):
+            monkeypatch.setattr(logging, 'raiseExceptions', False)
+            logging.warning('oops', 'first', 2)
+        """
+    )
+    result = testdir.runpytest()
+    result.assert_outcomes(passed=1)
+
+
+def test_log_file_cli_subdirectories_are_successfully_created(testdir):
+    path = testdir.makepyfile(""" def test_logger(): pass """)
+    expected = os.path.join(os.path.dirname(str(path)), "foo", "bar")
+    result = testdir.runpytest("--log-file=foo/bar/logf.log")
+    assert "logf.log" in os.listdir(expected)
+    assert result.ret == ExitCode.OK
