@@ -49,8 +49,16 @@ def test_disabled(testdir):
     assert result.ret == 0
 
 
-@pytest.mark.parametrize("enabled", [True, False])
-def test_timeout(testdir, enabled):
+@pytest.mark.parametrize(
+    "enabled",
+    [
+        pytest.param(
+            True, marks=pytest.mark.skip(reason="sometimes crashes on CI (#7022)")
+        ),
+        False,
+    ],
+)
+def test_timeout(testdir, enabled: bool) -> None:
     """Test option to dump tracebacks after a certain timeout.
     If faulthandler is disabled, no traceback will be dumped.
     """
@@ -71,8 +79,6 @@ def test_timeout(testdir, enabled):
 
     result = testdir.runpytest_subprocess(*args)
     tb_output = "most recent call first"
-    if sys.version_info[:2] == (3, 3):
-        tb_output = "Thread"
     if enabled:
         result.stderr.fnmatch_lines(["*%s*" % tb_output])
     else:
