@@ -1,7 +1,9 @@
+import argparse
 from typing import Optional
 
 import pytest
 from _pytest.config import ExitCode
+from _pytest.main import validate_basetemp
 from _pytest.pytester import Testdir
 
 
@@ -75,3 +77,24 @@ def test_wrap_session_exit_sessionfinish(
         assert result.ret == ExitCode.NO_TESTS_COLLECTED
     assert result.stdout.lines[-1] == "collected 0 items"
     assert result.stderr.lines == ["Exit: exit_pytest_sessionfinish"]
+
+
+@pytest.mark.parametrize("basetemp", ["foo", "foo/bar"])
+def test_validate_basetemp_ok(tmp_path, basetemp, monkeypatch):
+    monkeypatch.chdir(str(tmp_path))
+    validate_basetemp(tmp_path / basetemp)
+
+
+@pytest.mark.parametrize("basetemp", ["", ".", ".."])
+def test_validate_basetemp_fails(tmp_path, basetemp, monkeypatch):
+    monkeypatch.chdir(str(tmp_path))
+    msg = "basetemp must not be empty, the current working directory or any parent directory of it"
+    with pytest.raises(argparse.ArgumentTypeError, match=msg):
+        if basetemp:
+            basetemp = tmp_path / basetemp
+        validate_basetemp(basetemp)
+
+
+def test_validate_basetemp_integration(testdir):
+    result = testdir.runpytest("--basetemp=.")
+    result.stderr.fnmatch_lines("*basetemp must not be*")
