@@ -1,10 +1,13 @@
+from typing import List
+from typing import Union
+
 import pytest
 
 
 class TestPasteCapture:
     @pytest.fixture
-    def pastebinlist(self, monkeypatch, request):
-        pastebinlist = []
+    def pastebinlist(self, monkeypatch, request) -> List[Union[str, bytes]]:
+        pastebinlist = []  # type: List[Union[str, bytes]]
         plugin = request.config.pluginmanager.getplugin("pastebin")
         monkeypatch.setattr(plugin, "create_new_paste", pastebinlist.append)
         return pastebinlist
