@@ -6,6 +6,7 @@ import os
 import sys
 import textwrap
 from io import StringIO
+from typing import cast
 from typing import Dict
 from typing import List
 from typing import Tuple
@@ -14,10 +15,14 @@ import pluggy
 import py
 
 import _pytest.config
+import _pytest.terminal
 import pytest
+from _pytest._io.wcwidth import wcswidth
+from _pytest.config import Config
 from _pytest.config import ExitCode
 from _pytest.pytester import Testdir
 from _pytest.reports import BaseReport
+from _pytest.reports import CollectReport
 from _pytest.terminal import _folded_skips
 from _pytest.terminal import _get_line_with_reprcrash_message
 from _pytest.terminal import _plugin_nameversions
@@ -306,6 +311,29 @@ class TestTerminal:
         tr.rewrite("hey", erase=True)
         assert f.getvalue() == "hello" + "\r" + "hey" + (6 * " ")
 
+    def test_report_teststatus_explicit_markup(
+        self, testdir: Testdir, color_mapping
+    ) -> None:
+        """Test that TerminalReporter handles markup explicitly provided by
+        a pytest_report_teststatus hook."""
+        testdir.monkeypatch.setenv("PY_COLORS", "1")
+        testdir.makeconftest(
+            """
+            def pytest_report_teststatus(report):
+                return 'foo', 'F', ('FOO', {'red': True})
+        """
+        )
+        testdir.makepyfile(
+            """
+            def test_foobar():
+                pass
+        """
+        )
+        result = testdir.runpytest("-v")
+        result.stdout.fnmatch_lines(
+            color_mapping.format_for_fnmatch(["*{red}FOO{reset}*"])
+        )
+
 
 class TestCollectonly:
     def test_collectonly_basic(self, testdir):
@@ -330,17 +358,33 @@ class TestCollectonly:
         result = testdir.runpytest("--collect-only", "-rs")
         result.stdout.fnmatch_lines(["*ERROR collecting*"])
 
-    def test_collectonly_display_test_description(self, testdir):
+    def test_collectonly_displays_test_description(
+        self, testdir: Testdir, dummy_yaml_custom_test
+    ) -> None:
+        """Used dummy_yaml_custom_test for an Item without ``obj``."""
         testdir.makepyfile(
             """
             def test_with_description():
-                \""" This test has a description.
-                \"""
-                assert True
-        """
+                '''  This test has a description.
+
+                  more1.
+                    more2.'''
+            """
         )
         result = testdir.runpytest("--collect-only", "--verbose")
-        result.stdout.fnmatch_lines(["    This test has a description."])
+        result.stdout.fnmatch_lines(
+            [
+                "<YamlFile test1.yaml>",
+                "  <YamlItem test1.yaml>",
+                "<Module test_collectonly_displays_test_description.py>",
+                "  <Function test_with_description>",
+                "    This test has a description.",
+                "    ",
+                "    more1.",
+                "      more2.",
+            ],
+            consecutive=True,
+        )
 
     def test_collectonly_failed_module(self, testdir):
         testdir.makepyfile("""raise ValueError(0)""")
@@ -654,6 +698,29 @@ class TestTerminalFunctional:
         if request.config.pluginmanager.list_plugin_distinfo():
             result.stdout.fnmatch_lines(["plugins: *"])
 
+    def test_no_header_trailer_info(self, testdir, request):
+        testdir.monkeypatch.delenv("PYTEST_DISABLE_PLUGIN_AUTOLOAD")
+        testdir.makepyfile(
+            """
+            def test_passes():
+                pass
+        """
+        )
+        result = testdir.runpytest("--no-header")
+        verinfo = ".".join(map(str, sys.version_info[:3]))
+        result.stdout.no_fnmatch_line(
+            "platform %s -- Python %s*pytest-%s*py-%s*pluggy-%s"
+            % (
+                sys.platform,
+                verinfo,
+                pytest.__version__,
+                py.__version__,
+                pluggy.__version__,
+            )
+        )
+        if request.config.pluginmanager.list_plugin_distinfo():
+            result.stdout.no_fnmatch_line("plugins: *")
+
     def test_header(self, testdir):
         testdir.tmpdir.join("tests").ensure_dir()
         testdir.tmpdir.join("gui").ensure_dir()
@@ -662,10 +729,10 @@ class TestTerminalFunctional:
         result = testdir.runpytest()
         result.stdout.fnmatch_lines(["rootdir: *test_header0"])
 
-        # with inifile
+        # with configfile
         testdir.makeini("""[pytest]""")
         result = testdir.runpytest()
-        result.stdout.fnmatch_lines(["rootdir: *test_header0, inifile: tox.ini"])
+        result.stdout.fnmatch_lines(["rootdir: *test_header0, configfile: tox.ini"])
 
         # with testpaths option, and not passing anything in the command-line
         testdir.makeini(
@@ -676,12 +743,42 @@ class TestTerminalFunctional:
         )
         result = testdir.runpytest()
         result.stdout.fnmatch_lines(
-            ["rootdir: *test_header0, inifile: tox.ini, testpaths: tests, gui"]
+            ["rootdir: *test_header0, configfile: tox.ini, testpaths: tests, gui"]
         )
 
         # with testpaths option, passing directory in command-line: do not show testpaths then
         result = testdir.runpytest("tests")
-        result.stdout.fnmatch_lines(["rootdir: *test_header0, inifile: tox.ini"])
+        result.stdout.fnmatch_lines(["rootdir: *test_header0, configfile: tox.ini"])
+
+    def test_no_header(self, testdir):
+        testdir.tmpdir.join("tests").ensure_dir()
+        testdir.tmpdir.join("gui").ensure_dir()
+
+        # with testpaths option, and not passing anything in the command-line
+        testdir.makeini(
+            """
+            [pytest]
+            testpaths = tests gui
+        """
+        )
+        result = testdir.runpytest("--no-header")
+        result.stdout.no_fnmatch_line(
+            "rootdir: *test_header0, inifile: tox.ini, testpaths: tests, gui"
+        )
+
+        # with testpaths option, passing directory in command-line: do not show testpaths then
+        result = testdir.runpytest("tests", "--no-header")
+        result.stdout.no_fnmatch_line("rootdir: *test_header0, inifile: tox.ini")
+
+    def test_no_summary(self, testdir):
+        p1 = testdir.makepyfile(
+            """
+            def test_no_summary():
+                assert false
+        """
+        )
+        result = testdir.runpytest(p1, "--no-summary")
+        result.stdout.no_fnmatch_line("*= FAILURES =*")
 
     def test_showlocals(self, testdir):
         p1 = testdir.makepyfile(
@@ -1002,17 +1099,17 @@ def test_color_yes_collection_on_non_atty(testdir, verbose):
     assert "collected 10 items" in result.stdout.str()
 
 
-def test_getreportopt():
+def test_getreportopt() -> None:
     from _pytest.terminal import _REPORTCHARS_DEFAULT
 
-    class Config:
+    class FakeConfig:
         class Option:
             reportchars = _REPORTCHARS_DEFAULT
             disable_warnings = False
 
         option = Option()
 
-    config = Config()
+    config = cast(Config, FakeConfig())
 
     assert _REPORTCHARS_DEFAULT == "fE"
 
@@ -1439,6 +1536,21 @@ def test_terminal_summary_warnings_header_once(testdir):
     assert stdout.count("=== warnings summary ") == 1
 
 
+@pytest.mark.filterwarnings("default")
+def test_terminal_no_summary_warnings_header_once(testdir):
+    testdir.makepyfile(
+        """
+        def test_failure():
+            import warnings
+            warnings.warn("warning_from_" + "test")
+            assert 0
+    """
+    )
+    result = testdir.runpytest("--no-summary")
+    result.stdout.no_fnmatch_line("*= warnings summary =*")
+    result.stdout.no_fnmatch_line("*= short test summary info =*")
+
+
 @pytest.fixture(scope="session")
 def tr() -> TerminalReporter:
     config = _pytest.config._prepareconfig()
@@ -1587,7 +1699,7 @@ def test_summary_stats(
     class fake_session:
         testscollected = 0
 
-    tr._session = fake_session  # type: ignore[assignment]  # noqa: F821
+    tr._session = fake_session  # type: ignore[assignment]
     assert tr._is_last_item
 
     # Reset cache.
@@ -1953,7 +2065,7 @@ class TestProgressWithTeardown:
         output.stdout.re_match_lines([r"[\.E]{40} \s+ \[100%\]"])
 
 
-def test_skip_reasons_folding():
+def test_skip_reasons_folding() -> None:
     path = "xyz"
     lineno = 3
     message = "justso"
@@ -1962,35 +2074,32 @@ def test_skip_reasons_folding():
     class X:
         pass
 
-    ev1 = X()
+    ev1 = cast(CollectReport, X())
     ev1.when = "execute"
     ev1.skipped = True
     ev1.longrepr = longrepr
 
-    ev2 = X()
+    ev2 = cast(CollectReport, X())
     ev2.when = "execute"
     ev2.longrepr = longrepr
     ev2.skipped = True
 
     # ev3 might be a collection report
-    ev3 = X()
+    ev3 = cast(CollectReport, X())
     ev3.when = "collect"
     ev3.longrepr = longrepr
     ev3.skipped = True
 
-    values = _folded_skips([ev1, ev2, ev3])
+    values = _folded_skips(py.path.local(), [ev1, ev2, ev3])
     assert len(values) == 1
-    num, fspath, lineno, reason = values[0]
+    num, fspath, lineno_, reason = values[0]
     assert num == 3
     assert fspath == path
-    assert lineno == lineno
+    assert lineno_ == lineno
     assert reason == message
 
 
 def test_line_with_reprcrash(monkeypatch):
-    import _pytest.terminal
-    from wcwidth import wcswidth
-
     mocked_verbose_word = "FAILED"
 
     mocked_pos = "some::nodeid"
@@ -2014,8 +2123,8 @@ def test_line_with_reprcrash(monkeypatch):
     def check(msg, width, expected):
         __tracebackhide__ = True
         if msg:
-            rep.longrepr.reprcrash.message = msg
-        actual = _get_line_with_reprcrash_message(config, rep(), width)
+            rep.longrepr.reprcrash.message = msg  # type: ignore
+        actual = _get_line_with_reprcrash_message(config, rep(), width)  # type: ignore
 
         assert actual == expected
         if actual != "{} {}".format(mocked_verbose_word, mocked_pos):
@@ -2040,19 +2149,19 @@ def test_line_with_reprcrash(monkeypatch):
     check("some\nmessage", 80, "FAILED some::nodeid - some")
 
     # Test unicode safety.
-    check("😄😄😄😄😄\n2nd line", 25, "FAILED some::nodeid - ...")
-    check("😄😄😄😄😄\n2nd line", 26, "FAILED some::nodeid - ...")
-    check("😄😄😄😄😄\n2nd line", 27, "FAILED some::nodeid - 😄...")
-    check("😄😄😄😄😄\n2nd line", 28, "FAILED some::nodeid - 😄...")
-    check("😄😄😄😄😄\n2nd line", 29, "FAILED some::nodeid - 😄😄...")
+    check("🉐🉐🉐🉐🉐\n2nd line", 25, "FAILED some::nodeid - ...")
+    check("🉐🉐🉐🉐🉐\n2nd line", 26, "FAILED some::nodeid - ...")
+    check("🉐🉐🉐🉐🉐\n2nd line", 27, "FAILED some::nodeid - 🉐...")
+    check("🉐🉐🉐🉐🉐\n2nd line", 28, "FAILED some::nodeid - 🉐...")
+    check("🉐🉐🉐🉐🉐\n2nd line", 29, "FAILED some::nodeid - 🉐🉐...")
 
     # NOTE: constructed, not sure if this is supported.
-    mocked_pos = "nodeid::😄::withunicode"
-    check("😄😄😄😄😄\n2nd line", 29, "FAILED nodeid::😄::withunicode")
-    check("😄😄😄😄😄\n2nd line", 40, "FAILED nodeid::😄::withunicode - 😄😄...")
-    check("😄😄😄😄😄\n2nd line", 41, "FAILED nodeid::😄::withunicode - 😄😄...")
-    check("😄😄😄😄😄\n2nd line", 42, "FAILED nodeid::😄::withunicode - 😄😄😄...")
-    check("😄😄😄😄😄\n2nd line", 80, "FAILED nodeid::😄::withunicode - 😄😄😄😄😄")
+    mocked_pos = "nodeid::🉐::withunicode"
+    check("🉐🉐🉐🉐🉐\n2nd line", 29, "FAILED nodeid::🉐::withunicode")
+    check("🉐🉐🉐🉐🉐\n2nd line", 40, "FAILED nodeid::🉐::withunicode - 🉐🉐...")
+    check("🉐🉐🉐🉐🉐\n2nd line", 41, "FAILED nodeid::🉐::withunicode - 🉐🉐...")
+    check("🉐🉐🉐🉐🉐\n2nd line", 42, "FAILED nodeid::🉐::withunicode - 🉐🉐🉐...")
+    check("🉐🉐🉐🉐🉐\n2nd line", 80, "FAILED nodeid::🉐::withunicode - 🉐🉐🉐🉐🉐")
 
 
 @pytest.mark.parametrize(
@@ -2089,6 +2198,12 @@ def test_collecterror(testdir):
     )
 
 
+def test_no_summary_collecterror(testdir):
+    p1 = testdir.makepyfile("raise SyntaxError()")
+    result = testdir.runpytest("-ra", "--no-summary", str(p1))
+    result.stdout.no_fnmatch_line("*= ERRORS =*")
+
+
 def test_via_exec(testdir: Testdir) -> None:
     p1 = testdir.makepyfile("exec('def test_via_exec(): pass')")
     result = testdir.runpytest(str(p1), "-vv")
