@@ -1,12 +1,24 @@
 import os
 import stat
 import sys
+from typing import Callable
+from typing import List
 
 import attr
 
 import pytest
 from _pytest import pathlib
+from _pytest.pathlib import cleanup_numbered_dir
+from _pytest.pathlib import create_cleanup_lock
+from _pytest.pathlib import make_numbered_dir
+from _pytest.pathlib import maybe_delete_a_numbered_dir
+from _pytest.pathlib import on_rm_rf_error
 from _pytest.pathlib import Path
+from _pytest.pathlib import register_cleanup_lock_removal
+from _pytest.pathlib import rm_rf
+from _pytest.tmpdir import get_user
+from _pytest.tmpdir import TempdirFactory
+from _pytest.tmpdir import TempPathFactory
 
 
 def test_tmpdir_fixture(testdir):
@@ -33,9 +45,6 @@ class FakeConfig:
 
 class TestTempdirHandler:
     def test_mktemp(self, tmp_path):
-
-        from _pytest.tmpdir import TempdirFactory, TempPathFactory
-
         config = FakeConfig(tmp_path)
         t = TempdirFactory(TempPathFactory.from_config(config))
         tmp = t.mktemp("world")
@@ -48,8 +57,6 @@ class TestTempdirHandler:
 
     def test_tmppath_relative_basetemp_absolute(self, tmp_path, monkeypatch):
         """#4425"""
-        from _pytest.tmpdir import TempPathFactory
-
         monkeypatch.chdir(tmp_path)
         config = FakeConfig("hello")
         t = TempPathFactory.from_config(config)
@@ -91,7 +98,6 @@ def test_mktemp(testdir, basename, is_ok):
     mytemp = testdir.tmpdir.mkdir("mytemp")
     p = testdir.makepyfile(
         """
-        import pytest
         def test_abs_path(tmpdir_factory):
             tmpdir_factory.mktemp('{}', numbered=False)
         """.format(
@@ -181,7 +187,6 @@ def test_tmpdir_fallback_tox_env(testdir, monkeypatch):
     monkeypatch.delenv("USERNAME", raising=False)
     testdir.makepyfile(
         """
-        import pytest
         def test_some(tmpdir):
             assert tmpdir.isdir()
     """
@@ -207,7 +212,6 @@ def test_tmpdir_fallback_uid_not_found(testdir):
 
     testdir.makepyfile(
         """
-        import pytest
         def test_some(tmpdir):
             assert tmpdir.isdir()
     """
@@ -223,8 +227,6 @@ def test_get_user_uid_not_found():
     user id does not correspond to a valid user (e.g. running pytest in a
     Docker container with 'docker run -u'.
     """
-    from _pytest.tmpdir import get_user
-
     assert get_user() is None
 
 
@@ -234,8 +236,6 @@ def test_get_user(monkeypatch):
     required by getpass module are missing from the environment on Windows
     (#1010).
     """
-    from _pytest.tmpdir import get_user
-
     monkeypatch.delenv("USER", raising=False)
     monkeypatch.delenv("USERNAME", raising=False)
     assert get_user() is None
@@ -245,8 +245,6 @@ class TestNumberedDir:
     PREFIX = "fun-"
 
     def test_make(self, tmp_path):
-        from _pytest.pathlib import make_numbered_dir
-
         for i in range(10):
             d = make_numbered_dir(root=tmp_path, prefix=self.PREFIX)
             assert d.name.startswith(self.PREFIX)
@@ -261,20 +259,16 @@ class TestNumberedDir:
     def test_cleanup_lock_create(self, tmp_path):
         d = tmp_path.joinpath("test")
         d.mkdir()
-        from _pytest.pathlib import create_cleanup_lock
-
         lockfile = create_cleanup_lock(d)
-        with pytest.raises(EnvironmentError, match="cannot create lockfile in .*"):
+        with pytest.raises(OSError, match="cannot create lockfile in .*"):
             create_cleanup_lock(d)
 
         lockfile.unlink()
 
-    def test_lock_register_cleanup_removal(self, tmp_path):
-        from _pytest.pathlib import create_cleanup_lock, register_cleanup_lock_removal
-
+    def test_lock_register_cleanup_removal(self, tmp_path: Path) -> None:
         lock = create_cleanup_lock(tmp_path)
 
-        registry = []
+        registry = []  # type: List[Callable[..., None]]
         register_cleanup_lock_removal(lock, register=registry.append)
 
         (cleanup_func,) = registry
@@ -293,10 +287,8 @@ class TestNumberedDir:
 
         assert not lock.exists()
 
-    def _do_cleanup(self, tmp_path):
+    def _do_cleanup(self, tmp_path: Path) -> None:
         self.test_make(tmp_path)
-        from _pytest.pathlib import cleanup_numbered_dir
-
         cleanup_numbered_dir(
             root=tmp_path,
             prefix=self.PREFIX,
@@ -310,12 +302,9 @@ class TestNumberedDir:
         print(a, b)
 
     def test_cleanup_locked(self, tmp_path):
+        p = make_numbered_dir(root=tmp_path, prefix=self.PREFIX)
 
-        from _pytest import pathlib
-
-        p = pathlib.make_numbered_dir(root=tmp_path, prefix=self.PREFIX)
-
-        pathlib.create_cleanup_lock(p)
+        create_cleanup_lock(p)
 
         assert not pathlib.ensure_deletable(
             p, consider_lock_dead_if_created_before=p.stat().st_mtime - 1
@@ -330,16 +319,14 @@ class TestNumberedDir:
         self._do_cleanup(tmp_path)
 
     def test_removal_accepts_lock(self, tmp_path):
-        folder = pathlib.make_numbered_dir(root=tmp_path, prefix=self.PREFIX)
-        pathlib.create_cleanup_lock(folder)
-        pathlib.maybe_delete_a_numbered_dir(folder)
+        folder = make_numbered_dir(root=tmp_path, prefix=self.PREFIX)
+        create_cleanup_lock(folder)
+        maybe_delete_a_numbered_dir(folder)
         assert folder.is_dir()
 
 
 class TestRmRf:
     def test_rm_rf(self, tmp_path):
-        from _pytest.pathlib import rm_rf
-
         adir = tmp_path / "adir"
         adir.mkdir()
         rm_rf(adir)
@@ -355,8 +342,6 @@ class TestRmRf:
 
     def test_rm_rf_with_read_only_file(self, tmp_path):
         """Ensure rm_rf can remove directories with read-only files in them (#5524)"""
-        from _pytest.pathlib import rm_rf
-
         fn = tmp_path / "dir/foo.txt"
         fn.parent.mkdir()
 
@@ -374,8 +359,6 @@ class TestRmRf:
 
     def test_rm_rf_with_read_only_directory(self, tmp_path):
         """Ensure rm_rf can remove read-only directories (#5524)"""
-        from _pytest.pathlib import rm_rf
-
         adir = tmp_path / "dir"
         adir.mkdir()
 
@@ -386,9 +369,7 @@ class TestRmRf:
 
         assert not adir.is_dir()
 
-    def test_on_rm_rf_error(self, tmp_path):
-        from _pytest.pathlib import on_rm_rf_error
-
+    def test_on_rm_rf_error(self, tmp_path: Path) -> None:
         adir = tmp_path / "dir"
         adir.mkdir()
 
@@ -398,32 +379,32 @@ class TestRmRf:
 
         # unknown exception
         with pytest.warns(pytest.PytestWarning):
-            exc_info = (None, RuntimeError(), None)
-            on_rm_rf_error(os.unlink, str(fn), exc_info, start_path=tmp_path)
+            exc_info1 = (None, RuntimeError(), None)
+            on_rm_rf_error(os.unlink, str(fn), exc_info1, start_path=tmp_path)
             assert fn.is_file()
 
         # we ignore FileNotFoundError
-        exc_info = (None, FileNotFoundError(), None)
-        assert not on_rm_rf_error(None, str(fn), exc_info, start_path=tmp_path)
+        exc_info2 = (None, FileNotFoundError(), None)
+        assert not on_rm_rf_error(None, str(fn), exc_info2, start_path=tmp_path)
 
         # unknown function
         with pytest.warns(
             pytest.PytestWarning,
             match=r"^\(rm_rf\) unknown function None when removing .*foo.txt:\nNone: ",
         ):
-            exc_info = (None, PermissionError(), None)
-            on_rm_rf_error(None, str(fn), exc_info, start_path=tmp_path)
+            exc_info3 = (None, PermissionError(), None)
+            on_rm_rf_error(None, str(fn), exc_info3, start_path=tmp_path)
             assert fn.is_file()
 
         # ignored function
         with pytest.warns(None) as warninfo:
-            exc_info = (None, PermissionError(), None)
-            on_rm_rf_error(os.open, str(fn), exc_info, start_path=tmp_path)
+            exc_info4 = (None, PermissionError(), None)
+            on_rm_rf_error(os.open, str(fn), exc_info4, start_path=tmp_path)
             assert fn.is_file()
         assert not [x.message for x in warninfo]
 
-        exc_info = (None, PermissionError(), None)
-        on_rm_rf_error(os.unlink, str(fn), exc_info, start_path=tmp_path)
+        exc_info5 = (None, PermissionError(), None)
+        on_rm_rf_error(os.unlink, str(fn), exc_info5, start_path=tmp_path)
         assert not fn.is_file()
 
 
